/** 
 * 
 * @file TPIP3APP_load.cpp
 * @brief TPIP3 APP(APplication Program file) load program
 *
 * @author Katayama
 * @date 2015-05-12
 * @version 1.00 2015/05/12 katayama
 *
 * Copyright (C) 2015 TPIP User Community All rights reserved.
 * ̃t@C̒쌠́ATPIP[U[R~jeB̋Kɏ]
 * gp܂B
 */
#include <stdio.h>
#include "./include/TPFT_dll.h"
#pragma comment(lib, "./lib/TPFT3_10.lib")

/** initialize & load
 * @fn     int TPIP3APP_load(char* ip, char* prog_nm)
 * @param  ip      : TPIP3 IP address
 * @param  prog_nm : APP file name
 * @retval ==  1   : Already in running
 * @retval ==  0   : Start a successful
 * @retval == -1   : Function error
 * @retval == -2   : Parameter error
 * @retval == -3   : Communication error
 *
 */
int TPIP3APP_load(char* ip, char* prog_nm)
{
	int ans = 0;
	int err;
	
	err = TPFT_open(ip);
//	printf("TPFT_open()=%d \n", err);
	ans = -1;
	if (err ==  0)	ans =  0;	// OK
	if (err == -1)	ans = -3;	// Communication error
	if (err == -2) 	ans = -2;	// Parameter error
	if (err == -3)	ans = -3;	// Communication error
	
	if (ans == 0) {
		err = TPFT_check_process(prog_nm);
//		printf("TPFT_check_process() = %d \n", err);
		ans = -1;
		if (err ==  0)	ans =  1;	// Already in running
		if (err == -1)	ans =  0;	// OK
		if (err == -2)	ans = -2;	// Parameter error
		if (err == -3)	ans = -3;	// Communication error
		if (err == -4)	ans = -1;	// Function error
	}

	if (ans == 0) {
		err = TPFT_put(prog_nm, TPFT_READ|TPFT_EXECUTABLE);
//		printf("TPFT_put()=%d \n", err);
		ans = -1;
		if (err >=  0)	ans =  0;	// OK
		if (err == -1)	ans = -1;	// Function error
		if (err == -2)	ans = -2;	// Parameter error
		if (err == -3)	ans = -3;	// Communication error
	}

	if (ans == 0) {
		err = TPFT_run(prog_nm);
//		printf("TPFT_run()=%d \n", err);
		ans = -1;
		if (err ==  0)	ans =  0;	// OK
		if (err == -1)	ans = -1;	// Function error
		if (err == -2)	ans = -2;	// Parameter error
		if (err == -3)	ans = -3;	// Communication error
		if (err == -4)	ans = -1;	// Function error
	}

	if (ans == 0) {
		err = TPFT_check_process(prog_nm);
//		printf("TPFT_check_process() = %d \n", err);
		ans = -1;
		if (err ==  0)	ans =  0;	// OK
		if (err == -1)	ans = -1;	// Function error
		if (err == -2)	ans = -2;	// Parameter error
		if (err == -3)	ans = -3;	// Communication error
		if (err == -4)	ans = -1;	// Function error
	}
	return ans;
}

/** ending
 * @fn     int TPIP3APP_end(char* prog_nm)
 * @param  prog_nm : APP file name
 * @retval ==  0   : Successful
 * @retval == -1   : Function error
 * @retval == -2   : Parameter error
 * @retval == -3   : Communication error
 *
 */
int TPIP3APP_end(char* prog_nm)
{
	int ans = 0;
	int err;

	if (prog_nm) {
		err = TPFT_kill(prog_nm);
//		printf("TPFT_kill()=%d \n", err);
		ans = -1;
		if (err ==  0)	ans =  0;	// OK
		if (err == -1)	ans = -1;	// Function error
		if (err == -2)	ans = -2;	// Parameter error
		if (err == -3)	ans = -3;	// Communication error
	}

	err = TPFT_close();
//	printf("TPFT_close()=%d \n", err);
	if (ans == 0) {
		ans = -1;
		if (err ==  0)	ans =  0;	// OK
		if (err == -1)	ans = -1;	// Function error
	}

	return ans;
}
