/** 
 * 
 * @file    def_Share.h
 * @brief   Share memory Mapping ` header
 *
 * @author  Katayama
 * @date    2014-09-11
 * @version 1.00 2014/09/11
 * @version 1.01 2014/09/29
 * @version 1.02 2014/09/30
 * @version 1.10 2014/10/06
 * @version 1.20 2014/12/18 (def_rShare.h => def_Share.h)
 * @version 1.21 2015/02/05
 * @version 1.22 2015/04/02 (addition IMU reset)
 *
 * Copyright (C) 2014 TPIP User Community All rights reserved.
 * ̃t@C̒쌠́ATPIP[U[R~jeB̋Kɏ]
 * gp܂B
 */

// ver 1.01 game controller ǉ
// ver 1.02 monitor ǉ
// ver 1.10 {[h2Ή
// ver 1.20 IMUf[^ǉ
// ver 1.20 PI reset control ǉ
// ver 1.21 PCvOԋL̃f[^^Ao[ύX
// ver 1.21 TPIP3 - PC L(TPIP3 -> PC)̃f[^ǉ
// ver 1.22  PI reset controlIMUǉ

#ifndef __DEF_SHARE_H__
#define __DEF_SHARE_H__

#pragma pack(8)

/*
 *  ef[^\ `
 */

  // TPIP Xe[^X f[^\
typedef struct {
	unsigned pc      : 1;	// bit0 TPIP operation APP  (Ready = '1')
	unsigned tpip_v  : 1;	// bit1 TPIP video unit     (Ready = '1')
	unsigned tpip_c1 : 1;	// bit2 TPIP control unit#1 (Ready = '1')
	unsigned tpip_c2 : 1;	// bit3 TPIP control unit#2 (Ready = '1')
	unsigned no_use1 : 4;	// bit4 - 7  Unused
	unsigned imu     : 1;	// bit8 IMU  (Ready = '1')
	unsigned no_use2 : 23;	// bit9 - 31 Unused
} stat_bits_t;

typedef union {
	unsigned long  _dword;
	unsigned short _word[2];
	unsigned char  _byte[4];
	stat_bits_t    _bits;
} stat_t;


  // IMU f[^\
typedef struct {
	double gyro[3];  // Processed Gyro (x, y, z)
	double acc[3];   // Processed Accel (x, y, z)
	double mag[3];   // Processed Mag (x, y, z)
	double quat[4];  // Quaternion (w, x, y, z)
} imu_dt_t;

  // PI reset control
typedef union {
	long     _dword;
	struct {
	unsigned ch1    : 1;	// b0: ch1
	unsigned ch2    : 1;	// b1: ch2
	unsigned ch3    : 1;	// b2: ch3
	unsigned ch4    : 1;	// b3: ch4
	unsigned ch5    : 1;	// b4: ch5
	unsigned ch6    : 1;	// b5: ch6
	unsigned ch7    : 1;	// b6: ch7
	unsigned ch8    : 1;	// b7: ch8
	unsigned imu    : 1;	// b8: IMU
	unsigned no_use : 23;	// bit 9 - 31 Unused
	} s;
} pi_reset_t;



/*
 *  TPIP3 - PC L
 */

  // PC -> TPIP3 f[^\
typedef struct {
	unsigned short  d_out;		// DO bit3 - 0, 7 - 4
	short			pos[8];		// position ch1-8
	int             axis[8];	// game controller AXIS
	int             BTN;		// game controller button

	// PI control
	pi_reset_t      PI_reset;	// '0' => '1' b0:ch1, b1:ch2, b2:ch3, b3:ch4 
	unsigned int    key_code;	// key down code 32bit
} pc2tpip_dt_t;

  // TPIP3 -> PC f[^\ 
typedef struct {
	stat_t          status;		// TPIP status
								// --- sensor data
	unsigned short  AI[16];		// AI ch1-16
	long            PI[8];		// PI ch1-4
	unsigned short  batt;		// Battery monitor(0 - 3000)
	unsigned short  DI;			// bit 3 - 0, 7 - 4

								// --- monitor data
	unsigned short  d_out;		// DO bit3 - 0, 7 - 4
	short           PWM[8];		// TPIP3 PWM output data

	imu_dt_t       imu;			// IMU data
} tpip2pc_dt_t;



/*
 *  PCvOԋL
 */

  //  (TPIP3 )
typedef struct {
	unsigned short  d_out;		// DO bit3 - 0, 7 - 4
	short			pos[8];		// position ch1-8
	int             axis[8];	// game controller AXIS
	int             BTN;		// game controller button

	// PI control
	pi_reset_t      PI_reset;	// '0' => '1' b0:ch1, b1:ch2, b2:ch3, b3:ch4 
} tpip_ope_t;

  //  (TPIP3 j^)
typedef struct {
	// --- sensor data
	unsigned short  AI[16];		// AI ch1-16
	long            PI[8];		// PI ch1-4
	unsigned short  batt;		// Battery monitor(0 - 3000)
	unsigned short  DI;			// bit 3 - 0, 7 - 4

	// --- monitor data
	unsigned short  d_out;		// DO bit3 - 0, 7 - 4
	short           PWM[8];		// TPIP3 PWM output data
} tpip_mon_t;

  // PCvO L f[^\
typedef union {
    struct WIN_SHARE_STR{			// Another Access
        int          is_new;		// Data status 
        stat_t       status;		// TPIP status
        imu_dt_t     imu;			// IMU data
        tpip_ope_t   tpip_ope;		// TPIP3 
        tpip_mon_t   tpip_mon;		// TPIP3 j^
    } s;
	char   buff[sizeof(struct WIN_SHARE_STR)];	// Byte Access
} tpip_share_t;


#define RSHARE_PORT (9700)						// rShare UDP port No
#define WIN_SHARE_NAME "TPIP_SHARE"				// windows shared memory name
#define WIN_SHARE_STATE (8)						// windows shared memory state data size [byte]
#define WIN_SHARE_DATA  (sizeof(tpip_share_t))	// windows shared memory data size [byte]

#pragma pack(8)

#endif /* ifndef __DEF_SHARE_H__ */

